#include <avr/io.h>
#include <avr/interrupt.h>
#include "main.h"
#include "key.h"
#include "time.h"


void key_init(void)
{
	KEY_DIR &= ~ALL_KEYS;                                               // Portrichtung setzen
	KEY_PORT |= ALL_KEYS;                                               // PullUp's Ein
}


///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed. Each pressed key is reported
// only once
//
uint8_t get_key_press( uint8_t key_mask )
{
	cli();                                    // read and clear atomic !
	key_mask &= key_press;                    // read key(s)
	key_press ^= key_mask;                    // clear key(s)
	sei();
	return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
// check if a key has been pressed long enough such that the
// key repeat functionality kicks in. After a small setup delay
// the key is reported beeing pressed in subsequent calls
// to this function. This simulates the user repeadiately
// pressing and releasing the key.
//
uint8_t get_key_rpt( uint8_t key_mask )
{
	cli();                                    // read and clear atomic !
	key_mask &= key_rpt;                      // read key(s)
	key_rpt ^= key_mask;                      // clear key(s)
	sei();
	return key_mask;
}
 
///////////////////////////////////////////////////////////////////
//
void process_keys( void )
{
	static uint8_t ct0, ct1;
	static uint16_t rpt;
	uint8_t i;

	i = key_state ^ ~KEY_PIN;               // key changed ?

	ct0 = ~( ct0 & i );                     // reset or count ct0
	ct1 = ct0 ^ (ct1 & i);                  // reset or count ct1
	i &= ct0 & ct1;                         // count until roll over ?
	key_state ^= i;                         // then toggle debounced state

	if( key_state & KEY2 )                  // Verriegelung KEY2
	{
		key_press = key_state & i;          // 0->1: key press detect, nicht "speichern"
		key_rpt &= ~REPEAT_MASK;            // key_repeat lschen
	}

	else key_press |= key_state & i;        // 0->1: key press detect, "speichern"

	if( (key_state & REPEAT_MASK) == 0 )    // check repeat function
	{
		rpt = REPEAT_START;                 // start delay
	}

	if( --rpt == 0 )
	{
		rpt = REPEAT_NEXT;                  // repeat delay
		key_rpt |= key_state & REPEAT_MASK;
	}
}
