// time.c
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include "main.h"
#include "time.h"
#include "key.h"
#include "lamp.h"


/************************************************************************/
/*			Interrupt Service Routine			*/
/************************************************************************/
ISR( TIMER1_COMPA_vect )
{
	static unsigned int alarm = 0;

#if F_CPU % DEBOUNCE
	OCR1A = F_CPU / DEBOUNCE - 1;		                                // compare DEBOUNCE - 1 times
#endif

	if( --prescaler == 0 )                                              // exact one second over
	{
		prescaler = DEBOUNCE;
		if( eesave_cnt > 1 ) eesave_cnt--;

#if F_CPU % DEBOUNCE			                                        // handle remainder
		OCR1A = F_CPU / DEBOUNCE + F_CPU % DEBOUNCE - 1;                // compare once per second
#endif

	}                                                                   // end if( --prescaler == 0 )

	if( prescaler == compare )                                          // exakt eine Sekunde (nach Setzen der Variable 'compare') vergangen
	{
		if( time && count_down ) time--;
		else
		{
			lamp_off();
			compare = 0;

			if( !time )
			{
				alarm = DEBOUNCE * 4;                                   // Alarm 4 Sekunden lang
				count_down = 0;                                         // stoppe Countdown
			}
		}
	}

	if( alarm )                                                         // ist 'alarm' mehr als Null
	{
		if( !( --alarm % 64 ) )                                         // jeder 64. Interrupt (16x pro Sekunde), 'alarm' wird pre-dekrementiert
		{
//            if( alarm & 512 ) IO_PORT ^= SUMMER;                        // alle halbe Sekunde Port-"togglen" erlauben
//		    else IO_PORT &= ~SUMMER;                                    // oder auf'Aus' setzen
		}
	}

//    else IO_PORT &= ~SUMMER;

	if( !( prescaler % 8 ) ) process_keys();
}                                                                       // end ISR


void timer_init ( void )
{
	TCCR1B = (1 << WGM12) ^ (1 << CS10);	                            // divide by 1
                				        	                            // clear on compare
	OCR1A = F_CPU / DEBOUNCE - 1;
	TCNT1 = 0;
	prescaler = DEBOUNCE;

	TIMSK = (1 << OCIE1A);
}
